from pyspark.sql import SparkSession
from pyspark.conf import SparkConf
from ier_processor.spark_job import PDFBatch


def init_local_sc():
    spark_conf = SparkConf()
    SparkSession.builder.config(conf=spark_conf)
    spark = SparkSession.builder.master("local").getOrCreate()
    spark_context = spark.sparkContext
    AWS_ACCESS_KEY = 'AKIAYX2DMZI4G6GH3G6I'
    AWS_SECRET_KEY_ID = 'lHKq3Ok2WY2mkeoOPzeFYIokGK026wEjyZJswSP0'
    spark_context._jsc.hadoopConfiguration().set("fs.s3a.access.key", AWS_ACCESS_KEY)
    spark_context._jsc.hadoopConfiguration().set("fs.s3a.secret.key", AWS_SECRET_KEY_ID)

    spark_context._jsc.hadoopConfiguration().set("fs.s3a.endpoint", "s3.ap-south-1.amazonaws.com")
    spark_context._jsc.hadoopConfiguration().set("com.amazonaws.services.s3.enableV4", "true")

    spark_context._jsc.hadoopConfiguration().set("fs.s3a.impl", "org.apache.hadoop.fs.s3a.S3AFileSystem")
    spark_context._jsc.hadoopConfiguration().set("fs.s3a.aws.credentials.provider",
                                      "org.apache.hadoop.fs.s3a.BasicAWSCredentialsProvider")
    spark_context.setSystemProperty("com.amazonaws.services.s3.enableV4", "true")
    return spark_context

global_spark_context = init_local_sc()

def test_spark_job():
    test_batch = PDFBatch("Aldona", debug=True)
    #test_batch.install_ier_package(global_spark_context)
    print("extracting file names")
    test_batch.extract_pdf_file_names(global_spark_context)
    print("making pages")
    test_batch.make_pages(global_spark_context)
    print("extracting file names")
    test_batch.extract_page_image_file_names(global_spark_context)
    print("making records")
    test_batch.make_records(global_spark_context, debug=True)
    print("extracting file names")
    test_batch.extract_record_image_file_names(global_spark_context)
    print("making text records")
    test_batch.make_text_records(global_spark_context)



from ier_processor.pdf_slicing import PDFSlicingMachine
from ier_processor.image_record_processing import ImageProcessor
from pathlib import Path
import pytesseract
import os


def main():
    print("testing split functionality")
    slicer = PDFSlicingMachine(debug=True)
    test_pdf_file = os.path.join(Path(__file__).parent, 'test_data', 'whole_pdf', 'Benaulim', 'Part01.pdf')
    print(f"trying to split {test_pdf_file}..............")
    page_images = slicer.split_whole_pdf_into_pages(test_pdf_file, ".")
    for page_image in page_images:
        print(page_image)

    print('Now trying to further split one of the page images........')
    # we want to avoid page 2(this has photos etc.... )
    page_images.sort()
    #some_page = page_images[4]
    for i, some_page in enumerate(page_images):
        print(f'{i}..............Working with {some_page}')
        record_images = slicer.split_pdf_page_image_into_record_images(pdf_page_image_path=some_page, dest_base=".")
        for record_image in record_images:
            print(record_image)

    print("Now trying to extract text from one of the record images ......")
    some_record_image = record_images[8]
    print(f"Working with {some_record_image}")
    image_processor = ImageProcessor()

    record_type, extracted_data = image_processor.process_image(some_record_image)
    print(f'Image data is {record_type} :: {extracted_data}')

    assembly_image = list(filter(lambda x: "Assembly" in x, record_images))
    record_type, extracted_data = image_processor.process_image(assembly_image[0])
    print(f'Image data is {record_type} :: {extracted_data}')

    section_image = list(filter(lambda x: "Section" in x, record_images))
    record_type, extracted_data = image_processor.process_image(section_image[0])
    print(f'Image data is {record_type} :: {extracted_data}')





if __name__ == '__main__':
    if os.name == 'nt':
        pytesseract.pytesseract.tesseract_cmd = r'C:\Program Files\Tesseract-OCR\tesseract.exe'
    main()




import sys
from pyspark import SparkContext, SparkConf
from ier_processor.spark_job import PDFBatch

params = sys.argv
print("receive params", "\n".join(params))
batch = params[1] if len(params) > 1 else "Aldona"
action = params[2] if len(params) > 2 else "all"
debug = params[3] == 'True' if len(params) > 3 else True


# m5.xlarge has 4 vcps
# we have 5+ 1 instances
# cores is 20 = 5 * 4
print(f"batch:{batch} debug:{debug}")
#conf = SparkConf()
#conf.set("spark.executor.instances", 6)
#conf.set("spark.executor.cores", 2)
#conf.setAppName(f"PDFProcessing {batch}")

#sc = SparkContext(conf)
sc = SparkContext(appName=f"PDF Splitter debug:{debug}")
test_batch = PDFBatch(batch, debug=debug)


if action in ['all', 'extract_pdf_file_names']:
    print("extract_pdf_file_names")
    test_batch.extract_pdf_file_names(sc)
if action in ['all', 'make_pages']:
    print("make_pages")
    test_batch.make_pages(sc)
if action in ['all', 'extract_page_image_file_names']:
    print("extract_page_image_file_names")
    test_batch.extract_page_image_file_names(sc)
if action in ['all', 'make_records']:
    print("make_records")
    test_batch.make_records(sc)
if action in ['all', 'extract_record_image_file_names']:
    print("extract_record_image_file_names")
    test_batch.extract_record_image_file_names(sc)
if action in ['all', 'make_text_records']:
    print("make_text_records")
    test_batch.make_text_records(sc)
# separate section, and assembly from others




import re
from datetime import datetime
from urllib.parse import urlparse

import boto3

from ier_processor.image_record_processing import ImageProcessor
from ier_processor.pdf_slicing import PDFSlicingMachine


def strip_starting_slash(s):
    return re.sub('^/*', '', s)


class PDFBatch:

    BUCKET_BASE = "s3a://epicfiles"

    def __init__(self, name="Aldona", debug=True, local_spark=True):
        self.name = name
        self.debug = debug
        date_prefix = datetime.now().strftime('%Y%m%d_%H%M%s_')
        self.prefix = date_prefix if self.debug else ''
        self.local_spark = local_spark

    def proto(self):
        if self.local_spark:
            return 's3a'
        else:
            return 's3'

    @property
    def _pdf_files_all(self):
        return f'{PDFBatch.BUCKET_BASE}/GER_2020/{self.name}/*.pdf'

    @property
    def _pdf_files_sample(self):
        return f'{PDFBatch.BUCKET_BASE}/GER_2020/{self.name}/Part01.pdf'

    @property
    def pdf_files(self):
        if self.debug:
            return self._pdf_files_sample
        else:
            return self._pdf_files_all

    @property
    def pdf_file_names(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.name}/{self.prefix}/pdf_file_names/'

    @property
    def page_images(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.prefix}/page_images/'

    @property
    def page_image_names(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.name}/{self.prefix}/page_image_names/'

    @property
    def record_images(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.prefix}/record_images/'

    @property
    def record_image_names(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.name}/{self.prefix}/record_image_names/'

    @property
    def text_records(self):
        return f'{PDFBatch.BUCKET_BASE}/{self.prefix}/text_records/{self.name}/'

    def save_file_names_to(self, src_dir, dest_dir, global_spark_context):
        print(f'extracting file names from {src_dir} to {dest_dir}')

        print(f'deleting {dest_dir}')
        s3 = boto3.resource("s3")
        url_data = urlparse(dest_dir)
        bucket = s3.Bucket(url_data.netloc)
        key = strip_starting_slash(url_data.path)
        bucket.objects.filter(Prefix=key).delete()

        print(f'loading binary files from {src_dir}')
        binary_files = global_spark_context.binaryFiles(f'{src_dir}')
        file_names = binary_files.map(lambda binary_file_tuple: binary_file_tuple[0]).cache()
        file_names_count = file_names.count()

        print(f'saving list of {src_dir}::{file_names_count} to {dest_dir}')
        for file_name in file_names.take(100):
            print(f'                          {file_name}')
        file_names.saveAsTextFile(f'{dest_dir}/')
        file_names.unpersist()

    def extract_pdf_file_names(self, global_spark_context):
        try:
            self.save_file_names_to(self.pdf_files, self.pdf_file_names, global_spark_context)
        except Exception as e:
            pass

    def extract_page_image_file_names(self, global_spark_context):
        try:
            self.save_file_names_to(f'{self.page_images}/pdf_pages_as_images/{self.name}/*/*/*.png', self.page_image_names, global_spark_context)
        except Exception as e:
            pass

    def extract_record_image_file_names(self, global_spark_context):
        try:
            self.save_file_names_to(f'{self.record_images}/pdf_pages_as_images/{self.name}/*/*/*.png', self.record_image_names, global_spark_context)
        except Exception as e:
            pass

    def make_pages(self, global_spark_context):
        pdf_names = global_spark_context.textFile(f'{self.pdf_file_names}/*')
        print(f"                                          saving page images to {self.page_images}")
        limit = 1 if self.debug else None
        page_image_names = pdf_names.repartition(30).flatMap(lambda pdf_file: PDFSlicingMachine().split_whole_pdf_into_pages(pdf_file, self.page_images, limit=limit))
        print(page_image_names.count())
        for page_image in page_image_names.take(100):
            print(f"                {page_image}")

    def make_records(self, global_spark_context, debug=False):
        print('make records debug is ', debug)
        page_image_names = global_spark_context.textFile(f'{self.page_image_names}/*')
        image_records = page_image_names.repartition(30)
        if debug:
            image_records = global_spark_context.parallelize(image_records.takeSample(True, 3))
        image_records = image_records.flatMap(lambda pdf_page: PDFSlicingMachine().split_pdf_page_image_into_record_images(pdf_page, self.record_images))
        print(image_records.count())

    def make_text_records(self, global_spark_context):
        record_images = f'{self.record_images}/pdf_pages_as_images/{self.name}/*/*/*.png'
        record_images = global_spark_context.binaryFiles(record_images)
        text_records = record_images.map(lambda record_image: ImageProcessor().process_image(record_image[0], record_image[1])).cache()

        pdf_header_records = text_records.filter(lambda r: r[0] == 'PDFHeader').map(lambda r: r[1])
        assembly_records = text_records.filter(lambda r: r[0] == 'Assembly').map(lambda r: r[1])
        section_records = text_records.filter(lambda r: r[0] == 'Section').map(lambda r: r[1])
        records = text_records.filter(lambda r: r[0] == 'Record').map(lambda r: r[1])

        pdf_header_records.saveAsTextFile(self.text_records + "/pdf_header/")
        assembly_records.saveAsTextFile(self.text_records + "/assembly/")
        section_records.saveAsTextFile(self.text_records + "/section/")
        records.saveAsTextFile(self.text_records + "/records/")

        print(f'saving pdf_header records {pdf_header_records.count()} to {self.text_records + "/pdf_header/"}')
        [print("                  >", x) for x in pdf_header_records.take(10)]

        print(f'saving assembly records {assembly_records.count()} to {self.text_records + "/assembly/"}')
        [print("                  >", x) for x in assembly_records.take(10)]

        print(f'saving section records {section_records.count()} to {self.text_records + "/section/"}')
        [print("                  >", x) for x in section_records.take(10)]

        print(f'saving records {records.count()} to {self.text_records + "/records/"}')
        [print("                  >", x) for x in records.take(10)]


    def install_ier_package(self, global_spark_context):
        global_spark_context.install_pypi_package("indian-electoral-roll-processor")






import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README filinstall_requires e
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="indian-electoral-roll-processor",
    version="1.0.12",
    descp="Extract data from 2020 Indian Electoral Rolls ",
    long_descp=README,
    long_descp_content="text/markdown",
    URL="https://somesite.com",
    author="",
    authoremail="",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["ier_processor"],
    include_package_data=True,
    package_data={'ier_processor': ['test_data/whole_pdf/Benaulim/Part01.pdf']},
    install_requires=["asgiref==3.2.7",
                      "boto3==1.14.26",
                      "botocore==1.17.27",
                      "chardet==3.0.4",
                      "docutils==0.15.2",
                      "jmespath==0.10.0",
                      "pdf2image==1.13.1",
                      "Pillow==7.1.1",
                      "pyasn1==0.4.8",
                      "pycryptodome==3.9.7",
                      "pytesseract==0.3.3",
                      "python-dateutil==2.8.1",
                      "pytz==2019.3",
                      "PyYAML==5.3.1",
                      "rsa==4.5",
                      "s3transfer==0.3.3",
                      "six==1.14.0",
                      "urllib3==1.25.10",
                      ],

    entrypoints={
        "console_scripts": [
            "realpython=ier_processor.__main__:main",
        ]
    },
)

#data_files = [('test_data/whole_pdf/Benaulim', ['test_data/whole_pdf/Benaulim/Part01.pdf'])],
